"""
Test suite for the Observicia SDK.
"""

import os
import sys

# Add SDK root to Python path for test imports
sys.path.insert(0,
                os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
